// CoverSheet.java

/*
  This software uses the JAW (Java API Wrapper) library (see www.aplu.ch/jaw).
  It is Open Source Free Software, so you may
    - run the code for any purpose
    - study how the code works and adapt it to your needs
    - integrate all or parts of the code in your own programs
    - redistribute copies of the code
    - improve the code and release your improvements to the public
  However the use of the code is entirely your responsibility.

  Author: Aegidius Pluess, www.aplu.ch
*/

import ch.aplu.jaw.*;
import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.font.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;
import java.io.*;
import java.net.*;

public class CoverSheet extends JPanel
  implements NativeMouseListener, NativeKeyListener
{
  private final String version = "1.50 - June 2009";


  // Setup parameters
  private int xstart;
  private int ystart;
  private int border;
  private int xhome;
  private int yhome;
  private int opacity;
  private String color;
  private String activetitle;
  private boolean showLogo;
  private boolean enumwindows;
  private int[] colors = new int[3];

  interface State
  {
    int IDLE = 0;
    int MOVING = 1;
    int ENTERED = 2;
    int LEFT = 3;
    int QUITTING = 4;
  }

  private String info;
  private int width;
  private int height;

  private int xmin;
  private int xmax;
  private int ymin;
  private int ymax;

  private volatile int state = State.IDLE;
  private final String setupFile = "coversheet.xml";
  private JawGadget jg = new JawGadget(true);
  private JFrame frame;
  private TextWindow textWindow = null;
  private int xPos;
  private int yPos;
  private Color sheetColor;
  private Color textColor;
  private File bmp;
  private Color transparencyColor;
  private String logo = "CoverSheet";

  public CoverSheet()
  {
    getSetupParams();
    setInfo();
    Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
    width = dim.width - 2*border;
    height = dim.height - 2*border;
    setPreferredSize(new Dimension(width, height));
    xmin = -(width - border);
    xmax = width + border;
    ymin = -(height - border);
    ymax = height + border;
    xPos = xstart;
    yPos = ystart;
    frame = packComponent(this);

    jg.addNativeMouseListener(this, NativeMouse.lClick |
                                    NativeMouse.lRelease |
                                    NativeMouse.leave |
                                    NativeMouse.enter |
                                    NativeMouse.rClick);
    jg.addNativeKeyListener(this);

    sheetColor = new Color(colors[0],
                           colors[1],
                           colors[2]);

    if (Math.max(Math.max(colors[0], colors[1]), colors[2]) > 30)  // bright -> darker
      textColor = new Color((int)(0.8*colors[0]),
                            (int)(0.8*colors[1]),
                            (int)(0.8*colors[2]));
    else
      textColor = new Color((int)(1.3*(colors[0]+30)),  // dark -> brighter
                            (int)(1.3*(colors[1]+30)),
                            (int)(1.3*(colors[2]+30)));

    // Must be different than sheet and text color
    transparencyColor = new Color(colors[0] > 10 ? 0 : 255,
                                        colors[1],
                                        colors[2]);
    try
    {
      bmp = File.createTempFile("coversheet", ".bmp");
    }
    catch (IOException ex)
    {
      JOptionPane.showMessageDialog(
          null,
          "Can't create temporary file in system's temp directory",
          "Fatal Error",
          JOptionPane.INFORMATION_MESSAGE);
      System.exit(0);
    }

    jg.writeBMP(this, bmp);
    jg.showImage(bmp.getAbsolutePath(), opacity, transparencyColor, true, xPos, yPos);
    jg.addNativeKeyListener(this);

    if (enumwindows)
      textWindow = new TextWindow("Native Window Title Enumeration", jg.enumWindows());


    while (state != State.QUITTING)
    {
      switch (state)
      {
        case State.MOVING:
          jg.setPosition(xPos, yPos);
          break;
        case State.ENTERED:
          jg.activate();
          break;
        case State.LEFT:
          if (!activetitle.equals(""))
          {
            jg.activate(activetitle);
//            System.out.println("Activating: " + activetitle);
          }
          break;
      }
      putSleep();
    }
    if (textWindow != null)
      textWindow.dispose();
    bmp.delete();
    frame.dispose();
    jg.destroy();
  }

  // Callback methods
  public void mouseEvent(NativeMouse mouse)
  {
//    System.out.println("Event: " + mouse.getEvent() + " x: " + xPos + " y: " + yPos);
    switch (mouse.getEvent())
    {
      case NativeMouse.lClick:
        xPos = xhome;
        yPos = yhome;
        state = State.MOVING;
        wakeUp();
        break;

      case NativeMouse.enter:
        state = State.ENTERED;
        wakeUp();
        break;

      case NativeMouse.leave:
        state = State.LEFT;
        wakeUp();
        break;

      case NativeMouse.lRelease:
        xPos = jg.getX();
        yPos = jg.getY();
        break;

      case NativeMouse.rClick:
        showInfo();
        break;
    }
  }

  // Override paintComponent()
  public void paintComponent(Graphics g)
  {
    super.paintComponent(g);
    Graphics2D g2D = (Graphics2D)g;

    g2D.setColor(sheetColor);
    g2D.fillRect(0, 0, getSize().width, getSize().height);

    if (showLogo)
    {
      g2D.setColor(textColor);

      Font font = new Font("Arial", Font.ITALIC | Font.BOLD, 32);
      g2D.setFont(font);
      g2D.drawString(logo, 10, 30);

      font = new Font("Arial", Font.PLAIN, 16);
      g2D.setFont(font);
      g2D.drawString("  Right click for help", 10, 50);
    }
  }

  private JFrame packComponent(JComponent component)
  {
    JFrame f = new JFrame();
    f.getContentPane().add(component);
    f.pack();
    return f;
  }

  public void keyPressed(int keyCode, int modifier)
  {
//    System.out.println("code: " + keyCode + " modifier: " + modifier);
    int xNew = xPos;
    int yNew = yPos;
    int step = 10;
    boolean doMove = false;
    if (keyCode == 37 && modifier == 1)  // Ctrl-CursorLeft
    {
      if (xPos > xmin)
        xNew -= step;
    }
    if (keyCode == 39 && modifier == 1)  // Ctrl-CursorRight
    {
      if (xPos < xmax)
        xNew += step;
    }
    if (keyCode == 38 && modifier == 1)  // Ctrl-CursorUp
    {
      if (yPos > ymin)
        yNew -= step;
    }
    if (keyCode == 40 && modifier == 1)  // Ctrl-CursorDown
    {
      if (yPos < ymax)
        yNew += step;
    }
    if (keyCode == 36 && modifier == 1)  // Ctrl-Home
    {
      xNew = xhome;
      yNew = yhome;
    }
    if (keyCode == 33 && modifier == 1)  // Ctrl-PgUp
    {
      xNew = xstart;
      yNew = ystart;
    }
    if (keyCode == 34 && modifier == 1)  // Ctrl-PgDn
    {
      xNew = xstart;
      yNew = ymax;
    }
    if (keyCode == 112 && modifier == 1)  // Ctrl-F1
    {
      showInfo();
    }
    if (xNew != xPos || yNew != yPos)
    {
      xPos = xNew;
      yPos = yNew;
      jg.setPosition(xPos, yPos);
    }
  }

 private void putSleep()
  {
    synchronized(this)
    {
      try
      {
        wait();
      }
      catch (InterruptedException ex) {}
    }
  }

  private void wakeUp()
  {
    synchronized(this)
    {
      notify();
    }
  }

  private void showInfo()
  {
    jg.hideImage();
    Object[] options = { "OK", "CANCEL" };
    int rc =
    JOptionPane.showOptionDialog(null, info, "Quitting CoverSheet",
               JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
               null, options, options[0]);

    if (rc == JOptionPane.YES_OPTION)
    {
      state = State.QUITTING;
      wakeUp();
    }
    else
      jg.showImage(bmp.getAbsolutePath(), opacity, transparencyColor, true, xPos, yPos);
  }

  private void getSetupParams()
  {
    if (!((new File(setupFile)).exists()))
    {
      JOptionPane.showMessageDialog(null, "Cannot read file \"coversheet.xml\"",
                                  "Fatal Error",
                                  JOptionPane.INFORMATION_MESSAGE);
      System.exit(0);
    }

    MySAXParser msp = new MySAXParser();
    try
    {
      URL url = new File(setupFile).toURL();
      Object data[][] = msp.processURL(url);
      opacity = ((Integer)data[0][0]).intValue();
      checkRange("opacity", opacity, 0, 100);
      color = (String)data[0][1];
      xstart = ((Integer)data[0][2]).intValue();
      ystart = ((Integer)data[0][3]).intValue();
      border = ((Integer)data[0][4]).intValue();
      xhome = ((Integer)data[0][5]).intValue();
      yhome = ((Integer)data[0][6]).intValue();
      activetitle = (String)data[0][7];
      showLogo = ((Boolean)data[0][8]).booleanValue();
      enumwindows = ((Boolean)data[0][9]).booleanValue();
    }
    catch (Exception ex)
    {
      JOptionPane.showMessageDialog(null, "Illegal format detected in file \"coversheet.xml\"",
                                  "Fatal Error",
                                  JOptionPane.INFORMATION_MESSAGE);
      System.exit(0);
    }

    String[] colorsStr = color.split(",");
    try
    {
      for (int i = 0; i < 3; i++)
      {
        colors[i] = Integer.parseInt(colorsStr[i].trim());
        checkRange("color", colors[i], 0, 255);
      }
    }
    catch (NumberFormatException ex)
    {
      JOptionPane.showMessageDialog(null, "Illegal color format detected in file \"coversheet.xml\"",
                                  "Fatal Error",
                                  JOptionPane.INFORMATION_MESSAGE);
      System.exit(0);
    }
  }

  private void checkRange(String name, int value, int min, int max)
  {
    if (value < min || value > max)
    {
       JOptionPane.showMessageDialog(
          null,
          "Illegal value of \"" + name + "\" detected in file \"coversheet.xml\"" +
          "\nMust be in range (" + new Integer(min) + ".." + new Integer(max) + ")",
          "Fatal Error",
          JOptionPane.INFORMATION_MESSAGE);
      System.exit(0);
    }
  }

  private void setInfo()
  {
    info =
      "CoverSheet Version " + version + "\n" +
      "www.aplu.ch/coversheet\n\n" +
      "Keys: (when sheet has focus)\n" +
      "  Ctrl-Up:   Move up\n" +
      "  Ctrl-Dn:   Move down\n" +
      "  Ctrl-Lt:   Move left\n" +
      "  Ctrl-Rt:   Move right\n" +
      "  Ctrl-PgUp: Move to top\n" +
      "  Ctrl-PgDn: Move to bottom\n" +
      "  Ctrl-Home: Move to home position   \n" +
      "  Ctrl-F1:  Show this dialog\n" +
      "Mouse:\n" +
      "  LDrag: Move\n" +
      "  LClick: Move to home position\n" +
      "  RClick: Show this dialog\n" +
      "  Enter sheet: Get focus\n" +
      "Options: Edit file \"coversheet.xls\"\n\n" +
      "Quitting. Are you sure?";
  }

  public static void main(String args[])
  {
    new CoverSheet();
  }
}
